@@ -18,30 +18,83 @@ from os.path import exists, join
 import CharniakParser as parser
 import JohnsonReranker as reranker
 
-class Tree:
-    """Represents a single parse tree in Penn Treebank format. This
+class DeprecatedGetter:
+    """Used when a getter method has been converted to a property. All
+    attributes will be dispatched to the property's value and a warning
+    will be raised if this is called. This doesn't work if the property
+    being deprecated has its own __call__ method, since that will be
+    unreachable as a DeprecatedGetter."""
+    def __init__(self, name, value):
+        """name is the attribute name of the property. value is its
+        value."""
+        self.__name = name
+        self.__value = value
+    def __getattr__(self, attr):
+        """All attributes except __call__ are dispatched to the property's
+        value."""
+        return getattr(self.__value, attr)
+    def __call__(self, *args, **kwargs):
+        """Shouldn't be called except by deprecated code. Issues a warning
+        about the deprecation then returns the value so that deprecated
+        code will continue to work."""
+        from warnings import warn
+        warn("%r is no longer a method. It's now a property." % self.__name,
+             DeprecationWarning, stacklevel=2)
+        return self.__value
+
+class Tree(object):
+    """Represents a single parse (sub)tree in Penn Treebank format. This
     wraps the InputTree structure in the Charniak parser."""
     def __init__(self, input_tree_or_string):
         """These can be constructed from the Penn Treebank string
         representations of trees, e.g.:
 
             >>> Tree('(S1 (NP (NN tree)))')
-            bllipparser.RerankingParser.Tree('(S1 (NP (NN tree)))')
+            Tree('(S1 (NP (NN tree)))')
 
         Or from an existing InputTree (internal SWIG object). Users will
         generally want the former."""
         if not isinstance(input_tree_or_string, parser.InputTree):
             if not isinstance(input_tree_or_string, basestring):
-                raise TypeError("input_tree_or_string (%r) must be an InputTree or string." % input_tree_or_string)
+                raise TypeError("input_tree_or_string (%r) must be "
+                                "an InputTree or string." %
+                                input_tree_or_string)
             input_tree_or_string = \
                 parser.inputTreeFromString(input_tree_or_string)
         self._tree = input_tree_or_string
+        self._sd_tokens = None
+    def __getitem__(self, index):
+        """Indexes into subtrees for this node. Raises an IndexError if
+        there's no such subtree. Slices are supported."""
+        # list is necessary since otherwise it doesn't support all the
+        # slice stuff
+        subtrees = list(self._tree.subTrees())
+        try:
+            subtree = subtrees[index]
+        except IndexError:
+            if self.is_preterminal():
+                message = 'node is a preterminal'
+            else:
+                message = 'only %s children for this node' % len(self)
+            raise IndexError("list index %r out of range (%s)" %
+                             (index, message))
+        if isinstance(index, slice):
+            return [self.__class__(s) for s in subtree]
+        else: # single InputTree object
+            return self.__class__(subtree)
     def __iter__(self):
         """Provides an iterator over immediate subtrees in this Tree.
         Each item yielded will be a Tree object rooted at one of the
         children of this tree."""
         for tree in self._tree.subTrees():
             yield self.__class__(tree)
+    def all_subtrees(self):
+        """Iterates over all nodes in this tree in preorder (this node,
+        followed by its first child, etc.)"""
+        yield self
+        for subtree in self:
+            for subsubtree in subtree.all_subtrees():
+                yield subsubtree
     def subtrees(self):
         """Returns a list of direct subtrees."""
         return list(iter(self))
@@ -51,7 +104,7 @@ class Tree:
     def __repr__(self):
         """Provides a representation of this tree which can be used to
         reconstruct it."""
-        return '%s(%r)' % (self.__class__, str(self))
+        return '%s(%r)' % (self.__class__.__name__, str(self))
     def __str__(self):
         """Represent the tree in Penn Treebank format on one line."""
         return str(self._tree)
@@ -70,21 +123,123 @@ class Tree:
     def span(self):
         """Returns indices of the span for this tree: (start, end)"""
         return (self._tree.start(), self._tree.finish())
-    def label(self):
-        """Returns the label at the top of the tree. If the tree is a
-        preterminal, returns the part of speech."""
-        return self._tree.term()
+    def is_preterminal(self):
+        """Returns True iff this node is a preterminal (that is, its
+        label is a part of speech tag, it has a non-empty token, and it
+        has no child nodes)."""
+        return len(self) == 0
+
+    #
+    # properties
+    #
+
+    def token():
+        doc = """The word for the top node in this subtree. If this
+        node is not a preterminal, this will be return None. Setting
+        the token on a non-preterminal to anything other than None will
+        cause a ValueError. The same goes for setting a preterminal's
+        token to None."""
+        def fget(self):
+            return self._tree.word() or None
+        def fset(self, new_word):
+            new_word = new_word or None
+            if self.is_preterminal():
+                if new_word is None:
+                    raise ValueError("Can't set a null token on a "
+                                     "preterminal Tree.")
+            else:
+                # not a preterminal
+                if new_word is not None:
+                    raise ValueError("Can't set the token on a "
+                                     "non-preterminal Tree.")
+            self._tree.setWord(new_word)
+        return locals()
+    token = property(**token())
+
+    def label():
+        doc = """The label at the top of this subtree as a string. If
+        this tree is a preterminal, this will be its part of speech,
+        otherwise it will be the phrasal category. This property was
+        previously a method. It now returns a DeprecatedGetter as an
+        intermediate solution to help you find deprecated calls."""
+        def fget(self):
+            return DeprecatedGetter('label', self._tree.term())
+        def fset(self, new_label):
+            self._tree.setTerm(new_label)
+        return locals()
+    label = property(**label())
+
+    def label_suffix():
+        doc = """Suffix for the label at the top node of this subtree
+        (including the hyphen). These include function tags (e.g.,
+        "-SBJ" for subject") and coindexing ("-2"). In general, this
+        will be the empty string for any trees produced by BLLIP parser
+        but this property may be set if you read in gold trees."""
+        def fget(self):
+            return self._tree.ntInfo()
+        def fset(self, new_tag):
+            self._tree.setNtInfo(new_tag)
+        return locals()
+    label_suffix = property(**label_suffix())
+
+    def sd_tokens(self, sd_converter=None, conversion_kwargs=None):
+        """Convert this Tree to Stanford Dependencies
+        (requires PyStanfordDependencies). Returns a list of
+        StanfordDependencies.Token objects. This method caches
+        the converted tokens. You may optionally specify a
+        StanfordDependencies instance in sd_converter and keyword
+        arguments to StanfordDependencies.convert_tree as a dictionary
+        in conversion_kwargs."""
+        if not self._sd_tokens:
+            try:
+                import StanfordDependencies
+            except ImportError:
+                raise ImportError("For sd_tokens(), you need to install"
+                                  "PyStanfordDependencies from PyPI")
+            sd_converter = sd_converter or StanfordDependencies.get_instance()
+            conversion_kwargs = conversion_kwargs or {}
+            self._sd_tokens = sd_converter.convert_tree(str(self),
+                                                        **conversion_kwargs)
+        return self._sd_tokens
+
+    @classmethod
+    def trees_from_string(this_class, text):
+        """Given text containing multiple Penn Treebank trees, returns
+        a list of Tree objects (one for each tree in the text)."""
+        # Note: the native method below gives us memory ownership of
+        # the InputTree objects in the vector. We acquire their pointers
+        # and store them in a Python list (the vector won't stick
+        # around). InputTree objects typically contain other InputTree
+        # objects and the outer tree will free the inner trees when it is
+        # deleted. So, we only need (and want) to acquire the pointer of
+        # the outermost InputTree tree.
+        trees = list(parser.inputTreesFromString(text))
+        for tree in trees:
+            tree.this.acquire()
+        return map(this_class, trees)
+
+    @classmethod
+    def trees_from_file(this_class, filename):
+        """Given the path to a file containing multiple Penn Treebank
+        trees, returns a list of Tree objects (one for each tree in the
+        file)."""
+        # see trees_from_string for an explanation
+        trees = list(parser.inputTreesFromFile(filename))
+        for tree in trees:
+            tree.this.acquire()
+        return map(this_class, trees)
 
 class ScoredParse:
     """Represents a single parse and its associated parser
     probability and reranker score.
-    
+
     Properties:
     - ptb_parse: a Tree object representing the parse (str() it to get the
         actual PTB formatted parse)
     - parser_score: The log probability of the parse according to the parser
     - parser_rank: The rank of the parse according to the parser
-    - reranker_score: The log probability of the parse according to the reranker
+    - reranker_score: The log probability of the parse according to
+      the reranker
     - reranker_rank: The rank of the parse according to the reranker
 
     The latter two will be None if the reranker isn't being used."""
@@ -100,14 +255,19 @@ class ScoredParse:
             (self.parser_score, self.reranker_score, self.ptb_parse)
     def __repr__(self):
         return "%s(%r, parser_score=%r, reranker_score=%r)" % \
-            (self.__class__.__name__, str(self.ptb_parse), 
+            (self.__class__.__name__, str(self.ptb_parse),
              self.parser_score, self.reranker_score)
 
 class Sentence:
-    """Represents a single sentence as input to the parser. You should
-    not typically need to construct this object directly."""
+    """Represents a single sentence as input to the parser. Typically,
+    you won't need to construct this object directly. This wraps the
+    SentRep structure in the Charniak parser."""
     def __init__(self, text_or_tokens):
-        if isinstance(text_or_tokens, Sentence):
+        if isinstance(text_or_tokens, parser.SentRep):
+            # ensure that Python owns the pointer
+            text_or_tokens.this.acquire()
+            self.sentrep = text_or_tokens
+        elif isinstance(text_or_tokens, Sentence):
             self.sentrep = text_or_tokens.sentrep
         elif isinstance(text_or_tokens, basestring):
             self.sentrep = parser.tokenize('<s> ' + text_or_tokens + ' </s>')
@@ -115,8 +275,11 @@ class Sentence:
             # text_or_tokens is a sequence -- need to make sure that each
             # element is a string to avoid crashing
             text_or_tokens = [parser.ptbEscape(str(token))
-                for token in text_or_tokens]
+                              for token in text_or_tokens]
             self.sentrep = parser.SentRep(text_or_tokens)
+    def __repr__(self):
+        """Represent the Sentence as a string."""
+        return "%s(%s)" % (self.__class__, self.tokens())
     def __len__(self):
         """Returns the number of tokens in this sentence."""
         return len(self.sentrep)
@@ -127,6 +290,30 @@ class Sentence:
             tokens.append(self.sentrep.getWord(index).lexeme())
         return tokens
 
+    @classmethod
+    def sentences_from_string(this_class, text):
+        """Given text containing SGML(-ish) lines (typical input to
+        the command line parser), returns a list of Sentence objects
+        (one for each tree in the text). Example usage:
+
+        >>> Sentence.sentences_from_string('<s> Test </s>')
+        [bllipparser.RerankingParser.Sentence(['Test'])]
+        """
+        # Note that the native method below leaks. We work around this
+        # by acquiring its pointer in __init__
+        sentReps = parser.sentRepsFromString(text)
+        return map(this_class, sentReps)
+
+    @classmethod
+    def sentences_from_file(this_class, filename):
+        """Given the path to a filename containing multiple SGML(-ish)
+        lines (typical input to the command line parser), returns a list
+        of Sentence objects (one for each tree in the text)."""
+        # Note that the native method below leaks. We work around this
+        # by acquiring its pointer in __init__
+        sentReps = parser.sentRepsFromFile(filename)
+        return map(this_class, sentReps)
+
 class NBestList:
     """Represents an n-best list of parses of the same sentence."""
     def __init__(self, sentrep, parses, sentence_id=None):
@@ -136,9 +323,13 @@ class NBestList:
         self._sentrep = sentrep
         self.parses = []
         for index, (score, parse) in enumerate(parses):
+            # acquire the InputTree pointers or they'll never be freed
+            parse.this.acquire()
             scored_parse = ScoredParse(Tree(parse), score, parser_rank=index)
             self.parses.append(scored_parse)
         self.sentence_id = sentence_id
+        # True if we've added reranker scores to our parses
+        # (but doesn't necessarily imply that we're sorted by them)
         self._reranked = False
 
     def __getattr__(self, key):
@@ -146,8 +337,15 @@ class NBestList:
         return getattr(self.parses, key)
 
     def sort_by_reranker_scores(self):
-        self.parses.sort(key=lambda parse: -parse.reranker_score)
+        """Sort the parses by the reranker's score (highest to lowest).
+        If the reranker scores tie or there are no reranker scores, parser
+        probabilities are used as a secondary key."""
+        self.parses.sort(key=lambda parse: (parse.reranker_score,
+                                            parse.parser_score),
+                         reverse=True)
     def sort_by_parser_scores(self):
+        """Sort the parses by the parser's probability (most likely to least
+        likely)."""
         self.parses.sort(key=lambda parse: -parse.parser_score)
     def get_parser_best(self):
         """Get the best parse in this n-best list according to the parser."""
@@ -161,17 +359,17 @@ class NBestList:
     def tokens(self):
         """Get the tokens of this sentence as a sequence of strings."""
         return self._sentrep.tokens()
-    def rerank(self, reranker, lowercase=True):
-        """Rerank this n-best list according to a reranker model. reranker
-        can be a RerankingParser or RerankerModel."""
-        assert reranker
+    def rerank(self, reranker_instance, lowercase=True):
+        """Rerank this n-best list according to a reranker model.
+        reranker_instance can be a RerankingParser or RerankerModel."""
+        assert reranker_instance
         if not self.parses:
             self._reranked = True
             return
-        if isinstance(reranker, RerankingParser):
-            reranker = reranker.reranker_model
+        if isinstance(reranker_instance, RerankingParser):
+            reranker_instance = reranker_instance.reranker_model
         reranker_input = self.as_reranker_input()
-        scores = reranker.scoreNBestList(reranker_input)
+        scores = reranker_instance.scoreNBestList(reranker_input)
         # this could be more efficient if needed
         for (score, nbest_list_item) in zip(scores, self.parses):
             nbest_list_item.reranker_score = score
@@ -189,8 +387,9 @@ class NBestList:
             combined = StringIO()
             combined.write('%d %s\n' % (len(self.parses), sentence_id))
             for parse in self.parses:
-                combined.write('%s %s\n%s\n' % \
-                    (parse.reranker_score, parse.parser_score, parse.ptb_parse))
+                combined.write('%s %s\n%s\n' % (parse.reranker_score,
+                                                parse.parser_score,
+                                                parse.ptb_parse))
             combined.seek(0)
             return combined.read()
         else:
@@ -231,10 +430,11 @@ class RerankingParser:
         does not allow loading multiple models within the same process
         (calling this function twice will raise a RuntimeError)."""
         if self._parser_model_loaded:
-            raise RuntimeError('Parser is already loaded and can only be loaded once.')
+            raise RuntimeError('Parser is already loaded and can only '
+                               'be loaded once.')
         if not exists(model_dir):
-            raise ValueError('Parser model directory %r does not exist.' % \
-                model_dir)
+            raise ValueError('Parser model directory %r does not exist.' %
+                             model_dir)
         self._parser_model_loaded = True
         self.parser_model_dir = model_dir
         parser.loadModel(model_dir)
@@ -245,11 +445,11 @@ class RerankingParser:
         """Load the reranker model from its feature and weights files. A
         feature class may optionally be specified."""
         if not exists(features_filename):
-            raise ValueError('Reranker features filename %r does not exist.' % \
-                features_filename)
+            raise ValueError('Reranker features filename %r does not exist.' %
+                             features_filename)
         if not exists(weights_filename):
-            raise ValueError('Reranker weights filename %r does not exist.' % \
-                weights_filename)
+            raise ValueError('Reranker weights filename %r does not exist.' %
+                             weights_filename)
         self.reranker_model = reranker.RerankerModel(feature_class,
                                                      features_filename,
                                                      weights_filename)
@@ -266,7 +466,9 @@ class RerankingParser:
 
         sentence = Sentence(sentence)
         if len(sentence) > parser.max_sentence_length:
-            raise ValueError("Sentence is too long (%s tokens, maximum supported: %s)" % (len(sentence), parser.max_sentence_length))
+            raise ValueError("Sentence is too long (%s tokens, maximum "
+                             "supported: %s)" %
+                             (len(sentence), parser.max_sentence_length))
 
         try:
             parses = parser.parse(sentence.sentrep)
@@ -297,7 +499,7 @@ class RerankingParser:
             if isinstance(tags, basestring):
                 tags = [tags]
             tags = map(str, tags)
-            valid_tags = ext_pos.addTagConstraints(parser.VectorString(tags))
+            valid_tags = ext_pos.addTagConstraints(parser.StringVector(tags))
             if not valid_tags:
                 # at least one of the tags is bad -- find out which ones
                 # and throw a ValueError
@@ -314,8 +516,7 @@ class RerankingParser:
         """Helper method for just parsing a single sentence and getting
         its Penn Treebank tree.  If you want anything more complicated
         (e.g., the Tree objects, n-best lists, parser or reranker scores,
-        etc.), you'll want the more complicated parse() or parse_tagged()
-        interfaces.
+        etc.), you'll want the parse() or parse_tagged() interfaces.
 
             >>> rrp.simple_parse('Parse this.')
             '(S1 (S (VP (VB Parse) (NP (DT this))) (. .)))'
@@ -342,11 +543,12 @@ class RerankingParser:
         ext_pos = parser.ExtPos()
         bad_tags = set()
         for tag in set(tags):
-            good_tag = ext_pos.addTagConstraints(parser.VectorString([tag]))
+            good_tag = ext_pos.addTagConstraints(parser.StringVector([tag]))
             if not good_tag:
                 bad_tags.add(tag)
 
-        raise ValueError("Invalid POS tags (not present in the parser's terms.txt file): %s" % ', '.join(sorted(bad_tags)))
+        raise ValueError("Invalid POS tags (not present in the parser's "
+                         "terms.txt file): %s" % ', '.join(sorted(bad_tags)))
 
     def _check_loaded_models(self, rerank):
         """Given a reranking mode (True, False, 'auto') determines
@@ -355,7 +557,7 @@ class RerankingParser:
         value of rerank if rerank='auto')."""
         if not self._parser_model_loaded:
             raise ValueError("Parser model has not been loaded.")
-        if rerank == True and not self.reranker_model:
+        if rerank is True and not self.reranker_model:
             raise ValueError("Reranker model has not been loaded.")
         if rerank == 'auto':
             return bool(self.reranker_model)
@@ -363,7 +565,8 @@ class RerankingParser:
             return rerank
 
     def set_parser_options(self, language='En', case_insensitive=False,
-        nbest=50, small_corpus=True, overparsing=21, debug=0, smooth_pos=0):
+                           nbest=50, small_corpus=True, overparsing=21,
+                           debug=0, smooth_pos=0):
         """Set options for the parser. Note that this is called
         automatically by load_parser_model() so you should only need to
         call this to update the parsing options. The method returns a
@@ -384,10 +587,11 @@ class RerankingParser:
         parser to assign that value as the probability of seeing a known
         word in a new part-of-speech (one never seen in training)."""
         if not self._parser_model_loaded:
-            raise RuntimeError('Parser must already be loaded (call load_parser_model() first)')
+            raise RuntimeError('Parser must already be loaded (call '
+                               'load_parser_model() first)')
 
         parser.setOptions(language, case_insensitive, nbest, small_corpus,
-            overparsing, debug, smooth_pos)
+                          overparsing, debug, smooth_pos)
         self.parser_options = {
             'language': language,
             'case_insensitive': case_insensitive,
@@ -401,7 +605,7 @@ class RerankingParser:
 
     @classmethod
     def from_unified_model_dir(this_class, model_dir, parsing_options=None,
-        reranker_options=None, parser_only=False):
+                               reranker_options=None, parser_only=False):
         """Create a RerankingParser from a unified parsing model on disk.
         A unified parsing model should have the following filesystem
         structure:
@@ -422,15 +626,17 @@ class RerankingParser:
         (parser_model_dir, reranker_features_filename,
          reranker_weights_filename) = get_unified_model_parameters(model_dir)
         if parser_only and reranker_options:
-            raise ValueError("Can't set reranker_options if parser_only is on.")
+            raise ValueError("Can't set reranker_options if "
+                             "parser_only is on.")
 
         rrp = this_class()
         if parser_model_dir:
             rrp.load_parser_model(parser_model_dir, **parsing_options)
         if reranker_features_filename and reranker_weights_filename and \
-            not parser_only:
+           not parser_only:
             rrp.load_reranker_model(reranker_features_filename,
-                reranker_weights_filename, **reranker_options)
+                                    reranker_weights_filename,
+                                    **reranker_options)
 
         rrp.unified_model_dir = model_dir
         return rrp
