@@ -16,10 +16,10 @@ Python frontend to the BLLIP natural language parser.
 The BLLIP parser (also known as the Charniak-Johnson parser or
 Brown Reranking Parser) is described in the paper `Charniak
 and Johnson (Association of Computational Linguistics, 2005)
-<http://aclweb.org/anthology/P/P05/P05-1022.pdf>`_.  This package
+<http://aclweb.org/anthology/P/P05/P05-1022.pdf>`_. This package
 provides the BLLIP parser runtime along with a Python interface. Note
 that it does not come with any parsing models but includes a model
-downloader.  The primary maintenance for the parser takes place at
+downloader. The primary maintenance for the parser takes place at
 `GitHub <http://github.com/BLLIP/bllip-parser>`_.
 
 We request acknowledgement in any publications that make use of this
@@ -29,22 +29,24 @@ others to compare their results to yours.
 
 References:
 
-* Eugene Charniak and Mark Johnson. "Coarse-to-fine n-best parsing and
-  MaxEnt discriminative reranking." Proceedings of the 43rd Annual Meeting
-  on Association for Computational Linguistics. Association for
-  Computational Linguistics, 2005.
-  http://aclweb.org/anthology/P/P05/P05-1022.pdf
+* Eugene Charniak and Mark Johnson. "`Coarse-to-fine n-best parsing and
+  MaxEnt discriminative reranking
+  <http://aclweb.org/anthology/P/P05/P05-1022.pdf>`_."  Proceedings of
+  the 43rd Annual Meeting on Association for Computational Linguistics.
+  `Association for Computational Linguistics, 2005
+  <http://bllip.cs.brown.edu/publications/index_bib.shtml#charniak-johnson:2005:ACL>`_.
 
-* Eugene Charniak. "A maximum-entropy-inspired parser." Proceedings of
+* Eugene Charniak. "`A maximum-entropy-inspired parser
+  <http://aclweb.org/anthology//A/A00/A00-2018.pdf>`_." Proceedings of
   the 1st North American chapter of the Association for Computational
-  Linguistics conference. Association for Computational Linguistics, 2000.
-  http://aclweb.org/anthology/A/A00/A00-2018.pdf
+  Linguistics conference. `Association for Computational Linguistics, 2000
+  <http://bllip.cs.brown.edu/publications/index_bib.shtml#Charniak:2000:NAACL>`_.
 
 Fetching parsing models
 -----------------------
 
-Before you can parse, you'll need some parsing models.  ``ModelFetcher``
-will help you download and install parsing models.  It can be invoked
+Before you can parse, you'll need some parsing models. ``ModelFetcher``
+will help you download and install parsing models. It can be invoked
 from the command line. For example, this will download and install the
 standard WSJ model::
 
@@ -56,7 +58,7 @@ as a Python library::
 
     >>> from bllipparser.ModelFetcher import download_and_install_model
     >>> download_and_install_model('WSJ', '/tmp/models')
-    /tmp/models/WSJ
+    '/tmp/models/WSJ'
 
 In this case, it would download WSJ and install it to
 ``/tmp/models/WSJ``. Note that it returns the path to the downloaded
@@ -83,7 +85,7 @@ You can also load parser and reranker models manually::
 
     >>> rrp = RerankingParser()
     >>> rrp.load_parser_model('/tmp/models/WSJ/parser')
-    >>> rrp.load_reranker_model('/tmp/models/WSJ/reranker')
+    >>> rrp.load_reranker_model('/tmp/models/WSJ/reranker/features.gz', '/tmp/models/WSJ/reranker/weights.gz')
 
 If you only want the top parse of a sentence in Penn Treebank format, use
 the ``simple_parse()`` method::
@@ -92,15 +94,15 @@ the ``simple_parse()`` method::
     '(S1 (S (NP (DT This)) (VP (VBZ is) (ADJP (JJ simple))) (. .)))'
 
 If you want more information about the parse, you'll want to use the
-``parse()`` method which returns an ``NBestList`` object.  The parser
+``parse()`` method which returns an ``NBestList`` object. The parser
 produces an *n-best list* of the *n* most likely parses of the sentence
 (default: *n=50*). Typically you only want the top parse, but the others
 are available as well::
 
     >>> nbest_list = rrp.parse('This is a sentence.')
 
-To get information about the top parse (note that the ptb_parse property
-is a Tree object, described later)::
+To get information about the top parse (note that the ``ptb_parse``
+property is a ``Tree`` object, described in more detail later)::
 
     >>> print repr(nbest_list[0])
     ScoredParse('(S1 (S (NP (DT This)) (VP (VBZ is) (NP (DT a) (NN sentence))) (. .)))', parser_score=-29.621201629004183, reranker_score=-7.9273829816098731)
@@ -113,6 +115,24 @@ is a Tree object, described later)::
     >>> print len(nbest_list)
     50
 
+If you have the `PyStanfordDependencies
+<https://pypi.python.org/pypi/PyStanfordDependencies/>`_ package,
+you can parse straight to `Stanford Dependencies
+<http://nlp.stanford.edu/software/stanford-dependencies.shtml>`_::
+
+    >>> tokens = nbest_list[0].ptb_parse.sd_tokens()
+    >>> for token in tokens:
+    ...     print token
+    ...
+    Token(index=1, form=u'This', cpos=u'DT', pos=u'DT', head=4, deprel=u'nsubj')
+    Token(index=2, form=u'is', cpos=u'VBZ', pos=u'VBZ', head=4, deprel=u'cop')
+    Token(index=3, form=u'a', cpos=u'DT', pos=u'DT', head=4, deprel=u'det')
+    Token(index=4, form=u'sentence', cpos=u'NN', pos=u'NN', head=0, deprel=u'root')
+    Token(index=5, form=u'.', cpos=u'.', pos=u'.', head=4, deprel=u'punct')
+
+This will attempt to use a default converter but see docs for how to
+customize dependency conversion (or if you run into Java version issues).
+
 If you have an existing tokenizer, tokenization can also be specified
 by passing a list of strings::
 
@@ -172,7 +192,7 @@ There is an interactive shell which can help visualize a parse::
 
 Once in the shell, type a sentence to have the parser parse it::
 
-    rrp> I saw the astronomer with the telescope.
+    bllip> I saw the astronomer with the telescope.
     Tokens: I saw the astronomer with the telescope .
 
     Parser's parse:
@@ -192,11 +212,29 @@ Once in the shell, type a sentence to have the parser parse it::
 If you have ``nltk`` installed, you can use its tree visualization to
 see the output::
 
-    rrp> visual Show me this parse.
+    bllip> visual Show me this parse.
     Tokens: Show me this parse .
 
     [graphical display of the parse appears]
 
+If you have ``PyStanfordDependencies`` installed, you can parse straight
+to Stanford Dependencies::
+
+    bllip> sdparse Now with Stanford Dependencies integration!
+    Tokens: Now with Stanford Dependencies integration !
+
+    Parser and reranker:
+     Now [root]
+      +-- with [prep]
+      |  +-- integration [pobj]
+      |     +-- Stanford [nn]
+      |     +-- Dependencies [nn]
+      +-- ! [punct]
+
+The ``asciitree`` package is required to visualize Stanford Dependencies
+as a tree. If it is not available, the dependencies will be shown in
+CoNLL-X format.
+
 There is more detailed help inside the shell under the ``help`` command.
 
 The Tree class
@@ -230,39 +268,66 @@ Or get information about the labeled spans in the tree::
 
     >>> print tree.span()
     (0, 8)
-    >>> print tree.label()
+    >>> print tree.label
     S1
 
-And finally navigate within the trees::
+You can navigate within the trees and more::
 
     >>> tree.subtrees()
-    [bllipparser.RerankingParser.Tree('(S (NP (DT This)) (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree))) (. .))')]
-    >>> tree.subtrees()[0].label()
+    [Tree('(S (NP (DT This)) (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree))) (. .))')]
+    >>> tree[0] # first subtree
+    Tree('(S (NP (DT This)) (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree))) (. .))')
+    >>> tree[0].label
     'S'
-    >>> tree.subtrees()[0].subtrees()[0]
-    bllipparser.RerankingParser.Tree('(NP (DT This))')
-    >>> tree.subtrees()[0].subtrees()[0].label()
+    >>> tree[0][0] # first subtree of first subtree
+    Tree('(NP (DT This))')
+    >>> tree[0][0].label
     'NP'
-    >>> tree.subtrees()[0].subtrees()[0].span()
+    >>> tree[0][0].span()
     (0, 1)
-    >>> tree.subtrees()[0].subtrees()[0].tags()
+    >>> tree[0][0].tags()
     ('DT',)
-    >>> tree.subtrees()[0].subtrees()[0].tokens()
+    >>> tree[0][0].tokens() # tuple of all tokens in this span
     ('This',)
-    >>> len(tree.subtrees()[0]) # number of subtrees
+    >>> tree[0][0][0]
+    Tree('(DT This)')
+    >>> tree[0][0][0].token
+    'This'
+    >>> tree[0][0][0].label
+    'DT'
+    >>> tree[0][0][0].is_preterminal()
+    True
+    >>> len(tree[0]) # number of subtrees
     3
-    >>> for subtree in tree.subtrees()[0]:
+    >>> for subtree in tree[0]:
     ...    print subtree
     ... 
     (NP (DT This))
     (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree)))
     (. .)
+    >>> for subtree in tree.all_subtrees(): # all subtrees (recursive)
+    ...     print subtree.is_preterminal(), subtree
+    ...
+    False (S1 (S (NP (DT This)) (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree))) (. .)))
+    False (S (NP (DT This)) (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree))) (. .))
+    False (NP (DT This))
+    True (DT This)
+    False (VP (VBZ is) (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree)))
+    True (VBZ is)
+    False (NP (DT a) (ADJP (RB fairly) (JJ simple)) (NN parse) (NN tree))
+    True (DT a)
+    False (ADJP (RB fairly) (JJ simple))
+    True (RB fairly)
+    True (JJ simple)
+    True (NN parse)
+    True (NN tree)
+    True (. .)
 """
 
-from RerankingParser import RerankingParser, Tree, tokenize
+from RerankingParser import RerankingParser, Tree, Sentence, tokenize
 
 __authors__ = 'Eugene Charniak, Mark Johnson, David McClosky, many others'
 __license__ = 'Apache 2.0'
-__version__ = '2014.08.29b'
+__version__ = '2015.01.11'
 __maintainer__ = 'David McClosky'
 __email__ = 'notsoweird+pybllipparser@gmail.com'
